"""
command line interface definitions
"""
import pathlib
import subprocess
from typing import Dict, Union

import tomlkit
import typer

app = typer.Typer()

Requirement = Union[str, Dict[str, str]]
Dependencies = Dict[str, Requirement]


def get_pyproject_path(base_dir=pathlib.Path(".")) -> pathlib.Path:
    """
    Get the path to the pyproject.toml file in the given `base_dir` (which
    defailts to current directry)
    """
    return base_dir / "pyproject.toml"


def get_version(base_dir=pathlib.Path(".")) -> str:
    """get the version from pyproject.toml"""
    pyproject_path = get_pyproject_path(base_dir)

    pyproject = tomlkit.parse(pyproject_path.read_text())
    return pyproject["tool"]["poetry"]["version"]


def get_dependencies() -> Dependencies:
    """get the poerty dependencies from pyproject.toml"""
    pyproject_path = get_pyproject_path()

    pyproject = tomlkit.parse(pyproject_path.read_text())
    return pyproject["tool"]["poetry"]["dependencies"]


def set_changed_dependencies(changed_dependencies: Dependencies) -> None:
    """update the poerty dependencies in pyproject.toml"""
    pyproject_path = get_pyproject_path()

    # remove the changed dependencies
    pyproject = tomlkit.parse(pyproject_path.read_text())
    dep = pyproject["tool"]["poetry"]["dependencies"]
    for name in changed_dependencies:
        del dep[name]
    pyproject_path.write_text(tomlkit.dumps(pyproject))
    subprocess.call(["poetry", "update"])

    # make the actual change
    pyproject = tomlkit.parse(pyproject_path.read_text())
    dep = pyproject["tool"]["poetry"]["dependencies"]
    for name, req in changed_dependencies.items():
        if isinstance(req, dict):
            if list(req.keys()) == ["version"]:
                req = req["version"]
            else:
                new_req = tomlkit.inline_table()
                new_req.update(req)
                req = new_req

        dep[name] = req
    pyproject_path.write_text(tomlkit.dumps(pyproject))
    subprocess.call(["poetry", "update"])


@app.command()
def version():
    """Replace all path requirements with version requirements in pyproject.toml"""
    dependencies = get_dependencies()
    changed_dependencies = {}
    for name, req in dependencies.items():
        if isinstance(req, dict) and "path" in req and get_pyproject_path(pathlib.Path(req["path"])).exists():
            version_req = "^" + get_version(pathlib.Path(req["path"]))
            changed_dependencies[name] = dependencies[name]
            del changed_dependencies[name]["path"]
            changed_dependencies[name]["version"] = req["version"] = version_req
            typer.echo(f"{name}: Changing path requirement ../{name} to version requirement {req['version']}")

    set_changed_dependencies(changed_dependencies)


@app.command()
def path():
    """
    Replace all version dependencies with a path dependency in pyproject.toml (when ../dep_name exists)
    """
    dependencies = get_dependencies()
    changed_dependencies = {}
    for name, req in dependencies.items():
        if isinstance(req, str):
            dependencies[name] = req = {"version": req}

        if "path" not in req and get_pyproject_path(pathlib.Path("..") / name).exists():
            changed_dependencies[name] = dependencies[name]
            del changed_dependencies[name]["version"]
            changed_dependencies[name]["path"] = f"../{name}"
            typer.echo(f"{name}: Changing version requirement {req['version']} to path requirement../{name}")

    set_changed_dependencies(changed_dependencies)


if __name__ == "__main__":
    app()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['poetry_dev']

package_data = \
{'': ['*']}

install_requires = \
['tomlkit>=0.7.0,<0.8.0', 'typer>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['poetry_dev = poetry_dev:app']}

setup_kwargs = {
    'name': 'poetry-dev',
    'version': '0.1.2',
    'description': 'A collection of scripts replace local packages with versions and vice versa',
    'long_description': '# poetry-dev\n\nWhen developing multiple Python packages concurrently with Poetry manageed environments you\ncan install the local package as path requirements. Ie when you develop `bar`\nwhich have `foo` as dependency, which you also want to edit, you can\ndo `poetry add ../foo` from `bar` package. But when you want to publish\n`bar`, you have to change the path requirement back to a version requirement.\n\nAfter publishing `bar` you have to switch back to `foo` as a path requirement\nin order to continue develop both concurrently.\n\nThis package will help you to improve that task. With one command all version\nrequirements will be changed to path requirements (when the package is\ncheckout in a sibling directory with the same as the package name).\n\n`poetry_dev path`\n\nThis results in a changed `pyproject.toml` file. `poetry update` is called\nto make sure the package on the path is installed as editable package.\n\nBefore publishing, the path requirements can be switched back to version\nrequirements with the following command.\n\n`poetry_dev version`\n\nThe version of the dependency on the local path will be\nused as minimal caret version in the changed `pyproject.toml` and\n`poetry update` is called to make sure the corresponding version\nfrom the repository will be installed.',
    'author': 'Marc Rijken',
    'author_email': 'marc@rijken.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mrijken/poetry-dev',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
